"use strict";
/*
 * Copyright (c) 2018-2023, NeKz
 *
 * SPDX-License-Identifier: MIT
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.SourceDemoParser = exports.DefaultParsingOptions = void 0;
const buffer_js_1 = require("./buffer.js");
const demo_js_1 = require("./demo.js");
exports.DefaultParsingOptions = {
    header: true,
    messages: true,
    stringTables: false,
    dataTables: false,
    packets: false,
    userCmds: false,
};
class SourceDemoParser {
    constructor(options = exports.DefaultParsingOptions) {
        Object.defineProperty(this, "options", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.options = options;
    }
    static default() {
        return new this(exports.DefaultParsingOptions);
    }
    setOptions(options) {
        this.options = {
            ...this.options,
            ...options,
        };
        return this;
    }
    parse(buffer) {
        const extended = new Uint8Array(buffer.byteLength + 4 - (buffer.byteLength % 4));
        extended.set(new Uint8Array(buffer), 0);
        const buf = new buffer_js_1.SourceDemoBuffer(extended.buffer);
        const demo = demo_js_1.SourceDemo.default();
        if (this.options.header)
            demo.readHeader(buf);
        if (this.options.messages)
            demo.readMessages(buf);
        if (demo.messages?.length) {
            if (this.options.stringTables)
                demo.readStringTables();
            if (this.options.dataTables)
                demo.readDataTables();
            if (this.options.packets)
                demo.readPackets();
            if (this.options.userCmds)
                demo.readUserCmds();
        }
        return demo;
    }
}
exports.SourceDemoParser = SourceDemoParser;
